@@ -33,7 +33,7 @@ def cat(inputs, dim=0, output_device=None):
 
 class CatLinearOperator(LinearOperator):
     r"""
-    A `LinearOperator` that represents the concatenation of other lazy tensors.
+    A `LinearOperator` that represents the concatenation of other linear operators.
     Each LinearOperator must have the same shape except in the concatenating
     dimension.
 
@@ -244,6 +244,14 @@ class CatLinearOperator(LinearOperator):
                     res_list.append(res)
 
         elif torch.is_tensor(cat_dim_indices):
+            if cat_dim_indices.dim() > 1:
+                # Supporting broadcasting tensor indices for CatLinearOpeartor is very challenging to implement,
+                # and probably not a feature we need.
+                raise RuntimeError(
+                    "CatLinearOperator does not support broadcasted tensor indexes. "
+                    f"Got tensor indices of size {[idx.shape for idx in indices if torch.is_tensor(idx)]}."
+                )
+
             # Find out for which indices we switch to different tensors
             target_tensors = self.idx_to_tensor_idx[cat_dim_indices]
             does_switch_tensor = torch.ones(target_tensors.numel() + 1, dtype=bool_compat, device=self.device)
@@ -366,6 +374,9 @@ class CatLinearOperator(LinearOperator):
         )
         return res
 
+    def to_dense(self):
+        return torch.cat([to_dense(L) for L in self.linear_ops], dim=self.cat_dim)
+
     def inv_quad_logdet(self, inv_quad_rhs=None, logdet=False, reduce_inv_quad=True):
         res = super().inv_quad_logdet(inv_quad_rhs, logdet, reduce_inv_quad)
         return tuple(r.to(self.device) for r in res)
